#include <stdio.h>
#define __KERNEL__
#include <sys/types.h>
#include "disklabel.h"
#include <string.h>

#ifdef __osf__

typedef unsigned long u64;

#endif

int read_disklabel(int fd,struct disklabel *d) {
   if(lseek(fd,LABELPLACE,SEEK_SET)<0) {
      return -1;
   }
   if(read(fd,d,sizeof(*d))!=sizeof(*d)) {
      return -1;
   }
   if(d->d_magic!=DISKLABELMAGIC || d->d_magic2!=DISKLABELMAGIC) {
      fprintf(stderr,"Existing disk label is corrupt\n");
      return -1;
   }
   return 0;
}

int dosumlabel(int fd,struct disklabel *d) {
  u64 buf[128];
  int x;
  u64 sum=0;

  if(lseek(fd,0,SEEK_SET)<0) {
     return -1;
  }
  if(read(fd,buf,64*sizeof(u64))!=(64*sizeof(u64))) {
     return -1;
  }
  memcpy(&buf[LABELPLACE/sizeof(u64)],d,sizeof(*d));
  for(x=0;x<63;x++) {
    sum+=buf[x];
  }
  if(lseek(fd,63*sizeof(u64),SEEK_SET)<0) {
     return -1;
  }
  if(write(fd,&sum,sizeof(sum))!=sizeof(sum)) {
     return -1;
  }
  return 0;
}

static int does_overlap(int a1,int a2,int b1,int b2) {
   if(a1>b1 && a1<b2) {
      return 1;
   }
   if(a2>b1 && a2<b2) {
      return 1;
   }
   if(b1>a1 && b1<a2) {
      return 1;
   }
   if(b2>a1 && b2<a2) {
      return 1;
   }
   return 0;
}

int overlaplabel(struct disklabel *d,int offset,int endplace,int num) {
   int res=0;
   int x;

   for(x=0;x<d->d_npartitions;x++) {
      if(x!=num) {
	 int part_offset=d->d_partitions[x].p_offset;
	 int part_end=d->d_partitions[x].p_offset+d->d_partitions[x].p_size;
	 if(part_end>0 && does_overlap(part_offset,part_end,offset,endplace)) {
	    res=1;
	    if(num<0) {
	       fprintf(stderr,"Area overlaps with partition %d\n",x);
	    }
	    else {
	       fprintf(stderr,"Warning: added partition %d overlaps with partition %d\n",num,x);
	    }
	 }
      }
   }
   return res;
}

