#include <stdio.h>
#define __KERNEL__
#include <sys/types.h>
#include "disklabel.h"
#undef __KERNEL__
#include <fcntl.h>
#include <sys/stat.h>
#include <asm/system.h>

#define SECT_SIZE 512
#define BOOT_SECTOR 2

#ifdef __osf__

  typedef unsigned long u64;

#endif

main(argc,argv)
   int argc;
   char **argv;
{
   u64 bootsize,kernelsize=0;
   u64 bootsect=BOOT_SECTOR;
   u64 magicnum=0;
   int disk,file,kernel=0;
   struct disklabel dlabel;
   struct stat s;
   int x;
   char buf[2048];
   int verbose=1;
   
   if(argc<3) {
      fprintf(stderr,"Usage: %s disk bootfile [kernel]\n",argv[0]);
      exit(1);
   }
   disk=open(argv[1],O_RDWR);
   file=open(argv[2],O_RDONLY);
   if(disk<0 || file<0) {
      perror("open:");
      exit(1);
   }
   if(argc>3)
   {
     kernel=open(argv[3],O_RDONLY);
     if (kernel<0)
     {
       perror("open:");
       exit(1);
     }
   }
   if(read_disklabel(disk,&dlabel)) {
      fprintf(stderr,"Couldn't get a valid disk label, exiting\n");
      exit(1);
   }
   if(fstat(file,&s)) {
      perror("fstat");
      exit(1);
   }
   bootsize=(s.st_size+SECT_SIZE-1)/SECT_SIZE;
   if (kernel > 0)
   {
     if(fstat(kernel,&s)) {
       perror("fstat");
       exit(1);
     }
     kernelsize=(s.st_size+SECT_SIZE-1)/SECT_SIZE;
   }

   if(overlaplabel(&dlabel,bootsect,bootsize+bootsect+kernelsize,-1)) {
      fprintf(stderr,"I can't find anywhere to write the bootlabel\n");
      exit(1);
   }
   if(lseek(disk,60*8,SEEK_SET)<0) {
      perror("lseek on disk");
      exit(1);
   }
   write(disk,&bootsize,sizeof(bootsize));
   write(disk,&bootsect,sizeof(bootsect));
   write(disk,&magicnum,sizeof(magicnum));
   if (verbose)
   {
     fprintf(stderr,"bootsize:%lu sectors\n",bootsize);
     fprintf(stderr,"bootsect:%lu\n",bootsect);
     fprintf(stderr,"magic:%lu\n",magicnum);
   }
   if(lseek(disk,SECT_SIZE*bootsect,SEEK_SET)<0) {
      perror("lseek #2 on disk\n");
      exit(1);
   }
   while((x=read(file,buf,2048))>0) {
      write(disk,buf,x);
   }
   close(file);
   if (kernel > 0)
   {
     unsigned long len = 0;

     if (verbose)
       fprintf(stderr,"kernel:%lu sectors\n",kernelsize);
#if 0
     if(lseek(disk,BOOT_SIZE+BOOT_SECTOR*SECT_SIZE,SEEK_SET)<0) {
       perror("lseek #3 on disk\n");
       exit(1);
     }
#endif
     while((x=read(kernel,buf,2048))>0)
     {
       write(disk,buf,x);
       len += x;
     }
     close(kernel);
     if ((len+SECT_SIZE)/SECT_SIZE != kernelsize)
       fprintf(stderr,"warning: kernel read %lu, should be %lu\n",len/SECT_SIZE,kernelsize);
   }
   dosumlabel(disk,&dlabel);
   close(disk);
   exit(0);
}
